﻿using CommunityToolkit.Mvvm.Input;
using FlyADBase;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Net.Mail;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Web.UI.WebControls;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace AD2021.B2.TestApp
{
    /// <summary>
    /// PgFlyAd2021B2.xaml 的交互逻辑
    /// </summary>
    public partial class PgFlyAd2021B2 : Page
    {
        PgFlyAd2021B2Vm viewModel;
        public PgFlyAd2021B2()
        {
            InitializeComponent();
        }

        [Unity.InjectionMethod]
        public void Init(FlyAd2021B2 flyAd) 
        {
            viewModel = new PgFlyAd2021B2Vm();
            viewModel.Init(flyAd);
            this.DataContext = viewModel;
        }
    }

    public class PgFlyAd2021B2Vm : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        public UInt16 SetOStatusValue { get; set; }
        public bool IsAutoOutRunning { get; private set; }
        public string AreaWaitForAnswer { get; private set; }
        public int RuntoValue { get; set; }

        public bool IsAutoForwBackwEnable { get; set; }
        public int AutoForwBackwPos1 { get; set; }
        public int AutoForwBackwPos2 { get; set; }

        public UInt16 OStatus { get; set; }



        /// <summary>
        /// runto 速度 pps
        /// </summary>
        public UInt32 Velocity { get; set; }

        /// <summary>
        /// 开始速度 pps
        /// </summary>
        public UInt32 SVelocity { get; set; }

        /// <summary>
        /// 加速时间 ms
        /// </summary>
        public UInt32 ATime { get; set; }

        /// <summary>
        /// 减速时间 ms
        /// </summary>
        public UInt32 DTime { get; set; }

        /// <summary>
        /// 归0 第1段速 pps
        /// </summary>
        public UInt32 HVelocity1 { get; set; }

        /// <summary>
        /// 归0 第2段速 pps
        /// </summary>
        public UInt32 HVelocity2 { get; set; }



        public RelayCommand ConnectCmd { get; private set; }
        public RelayCommand DisconnectCmd { get; private set; }

        public RelayCommand SetOStatusCmd { get; private set; }

        public RelayCommand AutoOStatusCmd { get; private set; }

        public RelayCommand<string> SetBitCmd { get; private set; }

        public RelayCommand SetPosParamCmd { get; private set; }

        public RelayCommand RefreshParamCmd { get; private set; }

        public RelayCommand InitAreaCmd { get; private set; }
        public RelayCommand SetAccessCmd { get; private set; }

        public RelayCommand RuntoCmd { get; private set; }
        public RelayCommand Runto0Cmd { get; private set; }
        public RelayCommand RuntoPosLenCmd { get; private set; }
        public RelayCommand OrgCmd { get; private set; }
        public RelayCommand StopCmd { get; private set; }

        public RelayCommand BackwCmd { get; private set; }
        public RelayCommand ForwCmd { get; private set; }

        public RelayCommand OpenWdTimeGridCmd { get; private set; }
        public RelayCommand ResetCmd { get; private set; }

        private FlyAd2021B2 flyad;
        public FlyAd2021B2 FlyAd => flyad;

        DispatcherTimer autoForwBackwTimer;

        public PgFlyAd2021B2Vm() 
        {
            ConnectCmd = new RelayCommand(Connect);
            DisconnectCmd = new RelayCommand(Disconnect);
            SetOStatusCmd = new RelayCommand(SetOStatus);
            AutoOStatusCmd = new RelayCommand(AutoOStatus);
            SetBitCmd = new RelayCommand<string>(SetBit);
            InitAreaCmd = new RelayCommand(InitArea);
            SetAccessCmd = new RelayCommand(SetAccess);
            RuntoCmd = new RelayCommand(Runto);
            Runto0Cmd = new RelayCommand(Runto0);
            RuntoPosLenCmd = new RelayCommand(RuntoPosLen);
            OrgCmd = new RelayCommand(Org);
            StopCmd = new RelayCommand(Stop);
            BackwCmd = new RelayCommand(Backw);
            ForwCmd = new RelayCommand(Forw);
            SetPosParamCmd = new RelayCommand(SetPosParam);
            OpenWdTimeGridCmd = new RelayCommand(OpenWdTimeGrid);
            ResetCmd = new RelayCommand(Reset);
        }



        public void Init(FlyAd2021B2 flyAd)
        {
            this.flyad = flyAd;
            Misc.BindingOperations.SetBinding(this.flyad, nameof(flyad.OStatus), this, nameof(OStatus));
            Misc.BindingOperations.SetBinding(this.flyad, nameof(flyad.Velocity), this, nameof(Velocity));
            Misc.BindingOperations.SetBinding(this.flyad, nameof(flyad.SVelocity), this, nameof(SVelocity));
            Misc.BindingOperations.SetBinding(this.flyad, nameof(flyad.ATime), this, nameof(ATime));
            Misc.BindingOperations.SetBinding(this.flyad, nameof(flyad.DTime), this, nameof(DTime));
            Misc.BindingOperations.SetBinding(this.flyad, nameof(flyad.HVelocity1), this, nameof(HVelocity1));
            Misc.BindingOperations.SetBinding(this.flyad, nameof(flyad.HVelocity2), this, nameof(HVelocity2));


            this.PropertyChanged += PgFlyAd2021B2Vm_PropertyChanged;

            autoForwBackwTimer = new DispatcherTimer();
            autoForwBackwTimer.Interval = TimeSpan.FromSeconds(1);
            autoForwBackwTimer.Tick += AutoForwBackwTimer_Tick;
            
        }
        private void Reset()
        {
            flyad.Reset(0);
        }
        private void OpenWdTimeGrid()
        {
            WdTimeGrid w = new WdTimeGrid();
            w.Init(flyad);
            w.Show();
        }
        private void AutoForwBackwTimer_Tick(object sender, EventArgs e)
        {
            if (!flyad.IsReady)
                return;

            if (flyad.DriveStatus != DRIVE_MAN_STATUS.RUNNING) {
                //停下来了
                int d1 = Math.Abs(AutoForwBackwPos1 - flyad.Position);
                int d2 = Math.Abs(AutoForwBackwPos2 - flyad.Position);

                if (d1 < d2) 
                {
                    flyad.Runto(AutoForwBackwPos2);
                }
                else
                {
                    flyad.Runto(AutoForwBackwPos1);
                }
            }
        }

        private void PgFlyAd2021B2Vm_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == nameof(IsAutoForwBackwEnable)) 
            {
                if (IsAutoForwBackwEnable)
                {
                    autoForwBackwTimer.Start();
                }
                else {
                    autoForwBackwTimer.Stop();
                }
            }
        }

        private void SetPosParam()
        {
            flyad.SetPosParam(SVelocity, ATime, DTime, HVelocity1, HVelocity2);
            flyad.SetVelocity(Velocity);
        }


        private void Forw()
        {
            flyad.Forward();
        }

        private void Backw()
        {
            flyad.Backward();
        }

        private void Stop()
        {
            flyad.Stop();
        }

        private void Org()
        {
            flyad.Origin();
        }

        private void RuntoPosLen()
        {
            flyad.RuntoMax();
        }

        private void Runto0()
        {
            flyad.RuntoMin();
        }

        private void Runto()
        {
            flyad.Runto(RuntoValue);
        }

        private void SetAccess()
        {

        }

        private void InitArea()
        {

        }



        private void SetBit(string parameter)
        {
            string index_str = (string)parameter;
            if (!int.TryParse(index_str, out int index))
            {
                index = 0;
            }
            if (index < 0)
                index = 0;
            if (index > 15)
                index = 15;

            if (Misc.MyBase.CHECKBIT(flyad.OStatus, index))
            {
                flyad.SetOutputBit(index, false);
            }
            else {
                flyad.SetOutputBit(index, true);
            }
        }

        DispatcherTimer autoOStatusTimer;
        int autoOStatusIdx = 0;
        private void AutoOStatus()
        {
            if (IsAutoOutRunning)
            {
                IsAutoOutRunning = false;
                autoOStatusTimer.Stop();
                autoOStatusTimer.Tick -= AutoOStatusTimer_Tick;
                autoOStatusTimer = null;
            }
            else {
                IsAutoOutRunning = true;
                autoOStatusTimer = new DispatcherTimer();
                autoOStatusTimer.Tick += AutoOStatusTimer_Tick;
                autoOStatusTimer.Interval = TimeSpan.FromSeconds(1);
                autoOStatusTimer.Start();
            }
        }

        private void AutoOStatusTimer_Tick(object sender, EventArgs e)
        {
            if (autoOStatusIdx > 7)
                autoOStatusIdx = 0;
            flyad.SetOutput(0x00ff, (UInt16)~Misc.MyBase.BIT(autoOStatusIdx));
            autoOStatusIdx++;
        }

        private void SetOStatus()
        {
            flyad.SetOutput(0x00ff, SetOStatusValue);
        }

        private void Disconnect()
        {
            flyad.Disconnect();
        }

        private void Connect()
        {
            flyad.Connect();
        }


    }
}
