﻿using LiveCharts;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace AD2021.B2.TestApp
{
    /// <summary>
    /// Window1.xaml 的交互逻辑
    /// </summary>
    public partial class WdTimeGrid : Window
    {
        WdTimeGridVm viewModel;
        public WdTimeGrid()
        {
            InitializeComponent();
            this.Unloaded += WdTimeGrid_Unloaded;
        }

        private void WdTimeGrid_Unloaded(object sender, RoutedEventArgs e)
        {
            if (viewModel != null) {
                viewModel.Dispose();
                viewModel = null;
            }
        }

        [Unity.InjectionMethod]
        public void Init(FlyADBase.FlyAd2021B2 flyAd) 
        {
            viewModel = new WdTimeGridVm();
            viewModel.Init(flyAd);
            this.DataContext = viewModel;
        }
    }

    public class WdTimeGridVm : INotifyPropertyChanged,IDisposable
    {
        public event PropertyChangedEventHandler PropertyChanged;
        const int MaxCount = 2000;

        public ChartValues<int> Data1s { get; } = new ChartValues<int>();
        public ChartValues<int> Data2s { get; } = new ChartValues<int>();

        /// <summary>
        /// 间隔
        /// </summary>
        public int Interval { get; set; } = 1;

        List<int> Buf1 = new List<int>();
        List<int> Buf2 = new List<int>();

        List<int> Ad1s = new List<int>();
        List<int> Ad2s = new List<int>();
        FlyADBase.FlyAd2021B2 flyAd;
        public void Init(FlyADBase.FlyAd2021B2 flyAd) 
        {
            this.flyAd = flyAd;
            this.flyAd.TimeGridEvent += FlyAd_TimeGridEvent;
        }

        private void FlyAd_TimeGridEvent(object sender, FlyADBase.TimeGridEventArgs e)
        {
            //确保 Data 只有 2000个数据
            if (Interval <= 0)
            {
                Ad1s.AddRange(e.Ad1s);
                Ad2s.AddRange(e.Ad2s);
                Buf1.Clear();
                Buf2.Clear();
            }
            else
            {
                for (int i = 0; i < e.Ad1s.Count(); i++)
                {
                    Buf1.Add(e.Ad1s[i]);
                    if (Buf1.Count() >= Interval)
                    {
                        Ad1s.Add((int)Buf1.Average());
                        Buf1.Clear();
                    }
                    Buf2.Add(e.Ad2s[i]);
                    if (Buf2.Count() >= Interval)
                    {
                        Ad2s.Add((int)Buf2.Average());
                        Buf2.Clear();
                    }
                }
            }

            if (Ad1s.Count > MaxCount)
            {
                int removeCnt = Ad1s.Count - MaxCount;
                Ad1s.RemoveRange(0, removeCnt);
                Ad2s.RemoveRange(0, removeCnt);
            }

            Data1s.Clear();
            Data2s.Clear();

            Data1s.AddRange(Ad1s);
            Data2s.AddRange(Ad2s);
            return;
        }

        public void Dispose()
        {
            flyAd.TimeGridEvent -= FlyAd_TimeGridEvent;
        }
    }
}
