﻿using GeneralGommunication;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FlyXray
{
    public interface IFlyXrayCore : INotifyPropertyChanged
    {
        bool IsError { get; }

        string ErrMsg { get; }

        int ErrCnt { get; }

        bool IsConnected { get; }

        int ID { get; set; }

        /// <summary>
        /// 获取参数
        /// </summary>
        [CallBack(typeof(GetParam_Response))]
        void GetParam(CallBackHandler asyncDelegate, object asyncContext);


        /// <summary>
        /// 设置参数
        /// </summary>
        /// <param name="hvSet">高压设置</param>
        /// <param name="ianSet">管流设置</param>
        /// <param name="ilimSet">灯丝限制设置</param>
        /// <param name="hvMax">高压最大值</param>
        /// <param name="ianMax">管流最大值</param>
        /// <param name="ipreSet">灯丝预热设置</param>
        /// <param name="asyncDelegate"></param>
        /// <param name="asyncContext"></param>
        void SetParam(
            UInt32 hvSet,
            UInt32 ianSet,
            UInt32 ilimSet,
            UInt32 hvMax,
            UInt32 ianMax,
            UInt32 ipreSet,
            CallBackHandler asyncDelegate, object asyncContext);

        /// <summary>
        /// 保存参数
        /// </summary>
        /// <param name="asyncDelegate"></param>
        /// <param name="asyncContext"></param>
        void SaveParam(CallBackHandler asyncDelegate, object asyncContext);

        /// <summary>
        /// 获取当前值
        /// </summary>
        /// <param name="asyncDelegate"></param>
        /// <param name="asyncContext"></param>
        [CallBack(typeof(GetCurrentValue_Response))]
        void GetCurrentValue(CallBackHandler asyncDelegate, object asyncContext);



    }

    public class GetParam_Response
    {
        public UInt32 HvSet;
        public UInt32 IanSet;
        public UInt32 IlimSet;
        public UInt32 HvMax;
        public UInt32 IanMax;
        public UInt32 IpreSet;
    }
    public class GetCurrentValue_Response 
    {
        public UInt32 HvCurrent;
        public UInt32 IanCurrent;
    }
}
