﻿using FlyAdXBase;
using GalaSoft.MvvmLight.Command;
using LiveCharts;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Text.RegularExpressions;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Threading;

namespace FlyAdX.UI
{
    /// <summary>
    /// MainWindow.xaml 的交互逻辑
    /// </summary>
    public partial class MainWindow : Window
    {
        FlyAd2021MainVm viewModel;
        FlyAd2021ServiceClient flyAd2021;
        public MainWindow()
        {
            InitializeComponent();
            flyAd2021 = new FlyAd2021ServiceClient();
            flyAd2021.Init();
            viewModel = new FlyAd2021MainVm();
            viewModel.Init(flyAd2021, this, lbFuncMsg);
            this.DataContext = viewModel;

        }

        private void sendList_Selected(object sender, RoutedEventArgs e)
        {
            if (sendList.SelectedItem != null)
            {
                ReqMsg reqMsg = sendList.SelectedItem as ReqMsg;
                if (reqMsg != null)
                {
                    viewModel.ListToSend(reqMsg.Msg);
                }
            }
        }

        private void btnFuncClick(object sender, RoutedEventArgs e)
        {
            ReqMsg reqMsg = (sender as Button).Tag as ReqMsg;
            if (reqMsg != null)
            {
                viewModel.SendFuncPack(reqMsg.Msg);
            }

        }

        private void lbFuncMsg_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            viewModel.SelectedReply = (string)lbFuncMsg.SelectedItem;
        }

        private void TextBox_SelectionChanged(object sender, RoutedEventArgs e)
        {
            string msg = inputSelectedReply.SelectedText;
            if (string.IsNullOrWhiteSpace(msg))
                return;
            string[] hexs = msg.Split(' ');
            List<byte> array = new List<byte>();
            for (int i = 0; i < hexs.Count(); i++)
            {
                string hex = hexs[i];
                if (string.IsNullOrWhiteSpace(hex))
                    continue;
                if (!byte.TryParse(hex, System.Globalization.NumberStyles.HexNumber, null, out byte b))
                {
                    break;
                }
                array.Add(b);
            }
            string dectran = null;
            if (array.Count() == 1)
            {
                int value = (int)array[0];
                if (value >= 0)
                {
                    dectran = $"{value}";
                }
                else
                {
                    dectran = $"{value}({(uint)array[0]})";
                }
            }
            else if (array.Count() == 2)
            {
                int value = BitConverter.ToInt16(array.ToArray(), 0);
                if (value >= 0)
                {
                    dectran = $"{value}";
                }
                else
                {
                    dectran = $"{value}({BitConverter.ToUInt16(array.ToArray(), 0)})";
                }
            }
            else if (array.Count() == 4)
            {

                int value = BitConverter.ToInt32(array.ToArray(), 0);
                if (value >= 0)
                {
                    dectran = $"{value}";
                }
                else
                {
                    dectran = $"{value}({BitConverter.ToUInt32(array.ToArray(), 0)})";
                }
            }
            else
            {

            }
            viewModel.DecTran = dectran;
        }

        private void btnTranClick(object sender, RoutedEventArgs e)
        {
            TextBox_SelectionChanged(null, null);
        }
    }
    public class FlyAd2021MainVm : INotifyPropertyChanged
    {
        NLog.Logger dataPack_logger = NLog.LogManager.GetLogger("dataPack");
        NLog.Logger funcPack_logger = NLog.LogManager.GetLogger("funcPack");

        public event PropertyChangedEventHandler PropertyChanged;

        public DateTime Now { get; set; }
        public UInt16 AD { get; set; }
        public UInt16 AD2 { get; set; }
        public UInt16 ENC1 { get; set; }
        public UInt16 ENC2 { get; set; }
        public UInt16 In { get; set; }

        public UInt16 Out { get; set; }

        /// <summary>
        /// 限制最大显示 200 个数据
        /// </summary>
        public ChartValues<double> Values { get; private set; } = new ChartValues<double>();

        public ObservableCollection<ReqMsg> SendMsgList { get; } = new ObservableCollection<ReqMsg>();
        /// <summary>
        /// values容量
        /// </summary>
        public int ValuesCap { get; set; } = 1000;
        public bool IsUpdateValues { get; set; }
        public int Interval { get; set; } = 5;

        /// <summary>
        /// 数据包，调试记录
        /// </summary>
        public bool IsDataPackLog { get; set; }


        public ObservableCollection<string> FuncMsgList { get; } = new ObservableCollection<string>();

        public string FuncPack { get; set; }
        public RelayCommand StartCmd { get; private set; }
        public RelayCommand StopCmd { get; private set; }
        public RelayCommand SendFuncPackCmd { get; private set; }
        public RelayCommand ClearListCmd { get; private set; }
        public RelayCommand SaveSendMsgCmd { get; private set; }
        public RelayCommand<ReqMsg> DelSendMsgCmd { get; private set; }
        public RelayCommand<ReqMsg> EditSendMsgCmd { get; private set; }
        public RelayCommand<ReqMsg> MoveUpSendMsgCmd { get; private set; }
        public RelayCommand<ReqMsg> MoveDownSendMsgCmd { get; private set; }


        FlyAd2021ServiceClient flyAd2021;
        public FlyAd2021ServiceClient FlyAd { get { return flyAd2021; } }

        public string SelectedReply { get; set; }
        public string DecTran { get; set; }
        /// <summary>
        /// 保存剩余秒数
        /// </summary>
        public int SaveSurplusSec { get; private set; } = -1;


        DispatcherTimer timer;
        DispatcherTimer saveTimer;
        DispatcherObject dispatcherObject;
        ListBox lbFuncMsg;
        List<double> values = new List<double>();
        List<double> temps = new List<double>();
        public FlyAd2021MainVm()
        {
            StartCmd = new RelayCommand(Start);
            StopCmd = new RelayCommand(Stop);
            SendFuncPackCmd = new RelayCommand(SendFuncPack);
            SaveSendMsgCmd = new RelayCommand(SaveSendMsg);
            EditSendMsgCmd = new RelayCommand<ReqMsg>(EditSendMsg);
            DelSendMsgCmd = new RelayCommand<ReqMsg>(DelSendMsg);
            MoveUpSendMsgCmd = new RelayCommand<ReqMsg>(MoveUpSendMsg);
            MoveDownSendMsgCmd = new RelayCommand<ReqMsg>(MoveDownSendMsg);
            ClearListCmd = new RelayCommand(ClearList);
        }
        private void ClearList()
        {
            FuncMsgList.Clear();
        }

        public void Init(FlyAd2021ServiceClient flyAd2021, DispatcherObject dispatcherObject, ListBox lbFuncMsg)
        {
            this.flyAd2021 = flyAd2021;
            this.dispatcherObject = dispatcherObject;
            this.lbFuncMsg = lbFuncMsg;



            this.flyAd2021.TimeGridEvent += FlyAd2021_TimeGridEvent;

            this.flyAd2021.RecPackEvent += FlyAd2021_RecPackEvent;
            this.flyAd2021.FuncRecPackEvent += FlyAd2021_FuncRecPackEvent;
            this.flyAd2021.FuncSendPackEvent += FlyAd2021_FuncSendPackEvent;
            this.flyAd2021.DataRecPackEvent += FlyAd2021_DataRecPackEvent;


            MainJsonDb.Load(this, filePath);

            timer = new DispatcherTimer();
            timer.Tick += Timer_Tick;
            timer.Interval = TimeSpan.FromSeconds(0.1);


            saveTimer = new DispatcherTimer();
            saveTimer.Interval = TimeSpan.FromSeconds(1);
            saveTimer.Tick += SaveTimer_Tick;
            saveTimer.Start();
        }
        private void SaveTimer_Tick(object sender, EventArgs e)
        {
            if (SaveSurplusSec > 0)
            {
                SaveSurplusSec--;
            }
            else if (SaveSurplusSec == 0)
            {
                SaveSurplusSec--;
                //保存
                MainJsonDb.Save(this, filePath);
            }
        }
        
        
        
        private void FlyAd2021_DataRecPackEvent(object sender, PackEventArgs e)
        {

            if (IsDataPackLog)
            {
                string msg = $"数据包 {Misc.HexAndStr.Hex2Str2(e.Pack)}";
                dataPack_logger.Debug(msg);
            }
        }


        private void FlyAd2021_FuncSendPackEvent(object sender, PackEventArgs e)
        {
            {
                string msg = $"功能包 PC->AD盒 {Misc.HexAndStr.Hex2Str(e.Pack)}";
                funcPack_logger.Debug(msg);
            }
            dispatcherObject.Dispatcher.Invoke(() =>
            {
                string msg = $"{e.Time} PC->AD盒 {Misc.HexAndStr.Hex2Str2(e.Pack)}";
                FuncMsgList.Add(msg);
                msg = $"{e.Time} PC->AD盒 {Misc.HexAndStr.Hex2Str3(e.Pack)}";
                FuncMsgList.Add(msg);

                if (FuncMsgList.Count > 20)
                    FuncMsgList.RemoveAt(0);
                lbFuncMsg.ScrollIntoView(msg);

            });
        }

        private void FlyAd2021_FuncRecPackEvent(object sender, PackEventArgs e)
        {
            {
                string msg = $"功能包 AD盒->PC {Misc.HexAndStr.Hex2Str(e.Pack)}";
                funcPack_logger.Debug(msg);
            }

            dispatcherObject.Dispatcher.Invoke(() =>
            {
                string msg = $"{e.Time} AD盒->PC {Misc.HexAndStr.Hex2Str2(e.Pack)}";
                FuncMsgList.Add(msg);
                msg = $"{e.Time} AD盒->PC {Misc.HexAndStr.Hex2Str3(e.Pack)}";
                FuncMsgList.Add(msg);

                if (FuncMsgList.Count > 20)
                    FuncMsgList.RemoveAt(0);
                lbFuncMsg.ScrollIntoView(msg);
            });
        }

        private void FlyAd2021_RecPackEvent(object sender, PackEventArgs e)
        {
            //0.1刷新一次
        }
        bool GetAvg(UInt16 ad, out double avg)
        {
            temps.Add(ad);
            if (temps.Count() >= Interval)
            {
                avg = temps.Average(t => (double)t);
                temps.Clear();
                return true;
            }
            avg = 0;
            return false;
        }
        private void FlyAd2021_TimeGridEvent(object sender, TimeGridEventArgs e)
        {
            for (int i = 0; i < e.ADs.Count(); i++)
            {
                UInt16 ad = e.ADs[i];
                if (GetAvg(ad, out double avg))
                {
                    values.Add(avg);
                }
            }
            int remove_cnt = values.Count() - ValuesCap;
            if (remove_cnt > 0)
            {
                values.RemoveRange(0, remove_cnt);
            }
            if (IsUpdateValues)
            {
                Values.Clear();
                Values.AddRange(values);
            }
        }

        private void Timer_Tick(object sender, EventArgs e)
        {
            Now = flyAd2021.Now;
            AD = flyAd2021.AD;
            AD2 = flyAd2021.AD2;
            ENC1 = flyAd2021.ENC1;
            ENC2 = flyAd2021.ENC2;
            In = flyAd2021.In;
            Out = flyAd2021.Out;
        }

        private void Stop()
        {
            timer.Stop();
            flyAd2021.Stop();
        }

        private void Start()
        {
            temps.Clear();
            timer.Start();
            flyAd2021.Start();
        }


        private void MoveDownSendMsg(ReqMsg obj)
        {
            if (obj == null)
                return;
            int index = SendMsgList.IndexOf(obj);
            if (index < 0)
                return;
            if (index >= SendMsgList.Count() - 1)
                return;
            SendMsgList.Move(index, index + 1);
            SaveSurplusSec = 5;
        }

        private void MoveUpSendMsg(ReqMsg obj)
        {
            if (obj == null)
                return;
            int index = SendMsgList.IndexOf(obj);
            if (index < 0)
                return;
            if (index == 0)
                return;
            SendMsgList.Move(index, index - 1);
            SaveSurplusSec = 5;
        }

        private void DelSendMsg(ReqMsg reqMsg)
        {
            SendMsgList.Remove(reqMsg);
            SaveSurplusSec = 5;
        }

        private void SendFuncPack()
        {
            if (string.IsNullOrEmpty(FuncPack))
            {
                MessageBox.Show("没有任何数据");
                return;
            }

            //检测 ; 断开指令

            string[] funcpacks = FuncPack.Split(new string[] { ";", "\r\n" }, StringSplitOptions.RemoveEmptyEntries);
            List<List<byte>> bufs = new List<List<byte>>();
            for (int i = 0; i < funcpacks.Count(); i++)
            {
                string funcpack = funcpacks[i];
                if (string.IsNullOrWhiteSpace(funcpack))
                {
                    continue;
                }
                //翻译
                if (!Misc.HexAndStr.Str2Hex(funcpack, out List<byte> buf, out string errMsg))
                {

                    MessageBox.Show(funcpack + " " + errMsg);
                    return;
                }
                bufs.Add(buf);
            }

            if (bufs.Count() == 0)
            {
                MessageBox.Show("没有任何数据");
                return;
            }

            for (int i = 0; i < bufs.Count(); i++)
            {
                var buf = bufs[i];
                flyAd2021.Send(buf);
            }
        }

        public void ListToSend(string msg)
        {
            FuncPack = msg;
        }
        public void SendFuncPack(string msg)
        {
            ListToSend(msg);
            SendFuncPack();
        }

        string filePath = "wdmain.json";
        private void SaveSendMsg()
        {
            if (string.IsNullOrEmpty(FuncPack))
            {
                MessageBox.Show("没有任何数据");
                return;
            }

            //输入描述
            WdSaveSendMsg w = new WdSaveSendMsg();
            w.Init(FuncPack);
            if (w.ShowDialog() == true)
            {


                SendMsgList.Add(w.regMsg);
                SaveSurplusSec = 5;
            }
        }
        private void EditSendMsg(ReqMsg reqMsg)
        {
            //输入描述
            WdSaveSendMsg w = new WdSaveSendMsg();
            w.Init(reqMsg.Msg, reqMsg.Description);
            if (w.ShowDialog() == true)
            {
                reqMsg.Msg = w.regMsg.Msg;
                reqMsg.Description = w.regMsg.Description;
                SaveSurplusSec = 5;
            }
        }

    }

    public class ReqMsg : INotifyPropertyChanged
    {
        public string Msg { get; set; }
        public string Description { get; set; }

        public string Brief
        {
            get
            {
                if (string.IsNullOrEmpty(Description))
                    return null;
                int maxlen = 8;
                int len = Description.Length;
                if (len <= maxlen)
                    return Description;
                else
                    return Description.Substring(0, maxlen) + "...";
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;
    }

    public class MainJsonDb
    {
        public List<ReqMsg> ReqMsgs;

        public static bool Save(FlyAd2021MainVm src, string filePath)
        {
            MainJsonDb jsonDb = new MainJsonDb();
            jsonDb.ReqMsgs = src.SendMsgList.ToList();
            string json = Newtonsoft.Json.JsonConvert.SerializeObject(jsonDb, Newtonsoft.Json.Formatting.Indented);
            File.WriteAllText(filePath, json);
            return true;
        }
        public static bool Load(FlyAd2021MainVm src, string filePath)
        {
            if (!File.Exists(filePath))
                return false;
            string json = File.ReadAllText(filePath);
            try
            {
                var jsonDb = Newtonsoft.Json.JsonConvert.DeserializeObject<MainJsonDb>(json);
                src.SendMsgList.Clear();
                foreach (var r in jsonDb.ReqMsgs)
                    src.SendMsgList.Add(r);

                return true;
            }
            catch
            {
                return false;
            }

        }
    }
}

